/*
MeggyTwit
Copyright 2009 by Windell H. Oskay http://www.evilmadscientist.com/

An application designed to run in the Processing development environment, 
   version 1.0 or higher.   http://processing.org/

Big idea: Read recent activity on Twitter, display it on the Meggy Jr RGB.

Reads recent tweets by people that you follow will be sent over the serial interface
to the Meggy Jr.  To use this, you'll need to configure your username/pass, serial port, 
and (optionally) number of recent tweets to read, update frequency, scrolling rate, etc.



Additional libraries will be needed for this software to run properly,
the Twitter4j library ( http://yusuke.homeip.net/twitter4j/en/index.html )
and SimpleML library ( http://www.learningprocessing.com/tutorials/simpleML/ )

 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this library.  If not, see <http://www.gnu.org/licenses/>.

*/


// Required non-standard libraries: twitter4j, and simpleML.
// - Remember to restart Processing after installing new libraries.\

// Download SimpleML here: http://www.learningprocessing.com/tutorials/simpleML/ 
// The simpleML directory needs to go in a folder named "libraries" in your sketch folder. 

// Download twitter4j here: http://yusuke.homeip.net/twitter4j/en/index.html
// Easiest method to install it: extract the file, and use the Sketch>Add File...
// menu command and select the "twitter4j-2.0.8.jar" file. (Your version may vary).
// (That will add it to this particular Processing sketch, rather than install a library).
// If you know how to use it as a regular library, you can do that as well.


import simpleML.*;
//import twitter4j.*;
import processing.serial.*;
 
//  USER CONTROLS:
 
// This reads recent tweets of people that you follow.  
// Enter your Twitter user name and password, or this won't get very far at all....

String username = "YourScreenName"; // YOUR twitter username
String password = "********";       // YOUR twitter password.
  
// For serial communication to Meggy Jr RGB (or other Arduino-ish things!)
String SerialPort = "/dev/cu.usbserial-";     // Change to be the name of the port you want, e.g., COM5.

// Note 1: The default value  "/dev/cu.usbserial-";  is appropriate for Macs.
// Note 2: You do NOT need external hardware to run to run this example program.
// If no serial port is found matching the name that you have specified, this will still run in "simulation mode"
// that displays the twitter data in the Processing.org text output field.
  
 
float TwitterTimeDelay = 60;    // MINIMUM number of seconds between queries.

// At the time of writing this, the twitter LIMIT is 100 per hour!
// This limit has changed in the past; 
// see http://apiwiki.twitter.com/Rate-limiting for current info.

// Regardless, it will only check for new twitter activity AFTER it has finished scrolling through
// list of recent tweets, whose length is set by the variable, "HistoryLength"

int HistoryLength = 5;  // Number of recent tweets to display at a time.
// If set to 5, will display the 5 most recent tweets from your friends before checking for new activity. 
 
int ScrollSpeed = 8;    // frames per second to display on Meggy Jr: a number in the range 1 - 60. 
// You can also change this (while running) with the up/down arrow keys.



// END USER CONTROLS





int frames = ScrollSpeed; // "frame rate"
int TimeCt  = 0;

Serial myPort;
boolean SerialEnabled = false;    //Whether serial is enabled or not
boolean ComEstablished = false;   //Whether two-way communication is established to Meggy Jr
 
float TwitterMarkTime;  
 
String lasttweet;
String tweet;
String screenName; 
int AddSpace = 8;


Twitter twitter;
User[] friends;
java.util.List statuses = null;
int rate; // For twitter rate limiting
 
String[] TwitterTexts = new String[HistoryLength];
String[] TwitterNames = new String[HistoryLength];

int DisplayPhase = 2*HistoryLength;    // important initialiation. Don't screw with it.
//String DisplayString = " ";

byte CurrentColor = 2;
char StringPosition = 0;
char CharPosition = 0;

char[] SubChar = new char[5];
int letter_length = 1;
 
 
 // Optional strings for testing character sets
//String TESTString = "A B C D E F G H I J K L M N O P Q R S T U V W X Y Z ";
//String TESTString = "a b c d e f g h i j k l m n o p q r s t u v w x y z ";
//String TESTString = "1 2 3 4 5 6 7 8 9 0 - = ! @ # $ % ^ & * ( ) _ + { } [ ] \\ | : ; \' \" < > , . / ? ` ~ ";



void setup(){
  
  boolean OK = true;
  
  // Setup check: 
  
  if(username.equals("YourScreenName") == true) {
    
 println("\nUm... Dude? You totally forgot to enter your Twitter user name. I'm outa here.");
  exit();
  OK = false;
  }
   
  if (OK)
  if(password.equals("********") == true) {
 println("\nUm... Dude? Now, you totally forgot to enter your Twitter password. You can't fire me-- I quit!.");    
  exit();
  OK = false;
  }
  
  if (OK)
{
  // Graphics Setup:

  frameRate(frames);

  color bgColor;
  bgColor = color(70, 70, 200); 
  size(200, 200);
  smooth();  
  textFont(loadFont("Ziggurat.vlw"), 20); 
  background(bgColor);
  fill(255); 
  text("MeggyTwit!", 20, 50); 
  text("Click to Stop", 20, 90); 
  text("(Up/Down to", 15, 150); 
  text(" Change Speed)", 15, 175); 

  TwitterTimeDelay *= 1000; // convert seconds to milliseconds
  TwitterMarkTime =  millis() - TwitterTimeDelay ;

  // Twitter setup:

 
  twitter = new Twitter(username,password);
 
  lasttweet = " ";

  // Simplified standard serial port opening:
  //  myPort = new Serial(this, Serial.list()[1], 9600);


  // Improved Serial interface Setup:

  // Print a list of the serial ports, for debugging purposes:
  delay(50);

  int PortCount = Serial.list().length;
  int PortNumber = -1;

  // The default value is "/dev/cu.usbserial-"; which works on Macs.
  String str2; 

  int j = 0;
  while (j < (PortCount - 1)){
    str2 = Serial.list()[j].substring(0, 18);
    if(SerialPort.equals(str2) == true)
      PortNumber = j;

    j++; 
  }

  if (PortNumber >= 0)
  { 
    String portName = Serial.list()[PortNumber];
    myPort = new Serial(this, portName, 9600);
    myPort.buffer(1);
    myPort.clear();
    SerialEnabled = true; 
    println("Serial port "+Serial.list()[PortNumber]+" found and activated.");
  } 
  else
  {
    println("\nI found "+PortCount+" serial ports, which are:");
    println(Serial.list());
    println("But I don't know which one to use. :(\n");
    SerialEnabled = false; 
    println("Now entering serial-offline mode.\n");
  } 

}
else
{
  exit();
}

}

void draw() {

  byte xpos, ypos, colorNum;
  int TweetNoLocal;
String DisplayString2;


  /*
// Graphics & serial communication test routines.
   xpos = byte(floor(random(8)));
   ypos = byte(floor(random(8)));
   colorNum = byte(floor(random(14))); 
   
   serialDrawPixel(myPort,xpos,ypos,colorNum); 
   
   if (colorNum == 0)    // one in every 14 times... on average
   {
   serialSetAuxLEDs(int(floor(random(256)))); 
   serialShift(myPort);  //Shift Horiz, too.
   }
   
   if (byte(floor(random(100))) ==  0)
   serialErase(myPort);  //Erase Meggy screen
   */
 
  // Twitter data retrieval: Only after finishing display of all strings 

  if (DisplayPhase >= 2*HistoryLength)
  {    
    DisplayPhase = 0; 
    if ( millis() > (TwitterMarkTime + TwitterTimeDelay)){
      tweets();
      TwitterMarkTime =  millis();
    }
    else
    {
      TimeCt++;
      if (TimeCt > frames)
      { 
        print(".");   // waiting indicator
        TimeCt = 0;
      } 
    } 
  }
 
  char SubCharWord; 
  int StringChar;
  int FontChar = 0; 

  if (AddSpace > 0){

    serialShift(myPort);  //Shift Horiz by one unit
    AddSpace--;
  }

  else{
 
    //  StringChar = TESTString.charAt(StringPosition);      // For testing fonts
 

  TweetNoLocal = floor (DisplayPhase / 2);    // Tweet number to display
  
     if (DisplayPhase % 2  == 0)    // Display screen name, before the tweet
   {
       DisplayString2 = TwitterNames[TweetNoLocal];  
        
   }
    else
    {
    DisplayString2 = TwitterTexts[TweetNoLocal]; 
    }
    
      
    
    
    
    StringChar = DisplayString2.charAt(StringPosition);
     
    charLookup(StringChar);  

    SubCharWord = SubChar[CharPosition];


    CharPosition++;
    if (CharPosition >= letter_length)
    {
      CharPosition = 0;  // beginning of (next) character
      StringPosition++;  // move to next character

      AddSpace = 1;

      if (StringPosition >= DisplayString2.length()){
        StringPosition = 0; 
      DisplayPhase++;
      
  // Increment color:     
    if(++CurrentColor > 7)
       CurrentColor = 1;
 
      
      }
    }

    serialShift(myPort);  //Shift Horiz by one unit


      if ((SubCharWord & 1) > 0)
      serialDrawPixel(myPort, 7, 0,CurrentColor); 
    else
      serialDrawPixel(myPort,7,0,0); 

    if ((SubCharWord & 2) > 0)
      serialDrawPixel(myPort,7,1,CurrentColor); 
    else
      serialDrawPixel(myPort,7,1,0); 

    if ((SubCharWord & 4) > 0)
      serialDrawPixel(myPort,7,2,CurrentColor); 
    else
      serialDrawPixel(myPort,7,2,0); 

    if ((SubCharWord & 8) > 0)
      serialDrawPixel(myPort,7,3,CurrentColor); 
    else
      serialDrawPixel(myPort,7,3,0); 

    if ((SubCharWord & 16) > 0)
      serialDrawPixel(myPort,7,4,CurrentColor); 
    else
      serialDrawPixel(myPort,7,4,0); 

    if ((SubCharWord & 32) > 0)
      serialDrawPixel(myPort,7,5,CurrentColor); 
    else
      serialDrawPixel(myPort,7,5,0); 

    if ((SubCharWord & 64) > 0)
      serialDrawPixel(myPort,7,6,CurrentColor); 
    else
      serialDrawPixel(myPort,7,6,0); 

    if ((SubCharWord & 128) > 0)
      serialDrawPixel(myPort,7,7,CurrentColor); 
    else
      serialDrawPixel(myPort,7,7,0); 
  } 
}

void tweets(){
  int i;

  try
  {
    statuses = twitter.getFriendsTimeline(); 
  }
  catch( TwitterException f) 
  { 
    println(f.getStatusCode());
  }

  i = 0;
  while (i < HistoryLength)  
  {
    Status status = (Status)statuses.get(i); 
    tweet = status.getText(); 
    screenName =    status.getUser().getScreenName();


String[] TempFormatting = new String[2]; 
TempFormatting[0] = screenName; 
TempFormatting[1] = ": ";   

TwitterNames[i] = join(TempFormatting, ""); 

TempFormatting[0] = tweet; 
TempFormatting[1] = "     ";  

TwitterTexts[i] = join(TempFormatting, ""); 
  

    println(""); // return
    //  println(screenName +" writes:"); 
    println(screenName +"::"); 
    println(tweet); 
    
    i++;

  }
  println(""); // return  



    try
  {
    rate = twitter.rateLimitStatus().getRemainingHits();
    println("Twitter queries allowed this hour: " + rate);
  }
  catch( TwitterException e) 
  { 
    println(e.getStatusCode());
  }

  if (tweet.length() != lasttweet.length()){
    // println("==New Tweet==\n\n"); 

  }
  else
  {
    //  println("==No Activity==\n\n"); 
  }

  Status Lstatus = (Status)statuses.get(0);
  lasttweet = Lstatus.getText();
}
 

void mousePressed()
{ 
  SerialEnabled  = false;
  
    if (SerialEnabled)
  {  
  myPort.clear();
  myPort.stop();
  }
  
  exit();

} 


void keyPressed() {
  if (key == CODED) {
    if (keyCode == UP)
    {
      if (ScrollSpeed < 60)
      {
      ScrollSpeed++;
      }     
       frameRate(ScrollSpeed);
  
    } 
    else if (keyCode == DOWN) 
    {
      if (ScrollSpeed > 1)
      {
      ScrollSpeed--;
      }
       frameRate(ScrollSpeed);
      
    } 
  } 
}



void serialSetAuxLEDs(int value)
{
  if (SerialEnabled)
  {  
    if (value > 255)
      value = 255;

    myPort.write('a'); 
    myPort.write(value);  
    myPort.write('A'); 
  } 
  return;

}





boolean serialShift(Serial myPort)
{  
  if (SerialEnabled)
  {  
    myPort.write('s');  
    return true; 
  }
  else
    return false;
} // end serialShift





boolean serialDrawPixel(Serial myPort, int x, int y, int colorNumber)
{  
  if (SerialEnabled)
  {  
    myPort.write('d'); 
    myPort.write(x); 
    myPort.write(y); 
    myPort.write(colorNumber); 
    myPort.write('D'); 

    return true; 
  }
  else
    return false;
} // end serialDrawPixel





boolean serialDisplaySlate(Serial myPort)
{  
  if (SerialEnabled)
  {  
    myPort.write('s'); 
    return true; 
  }
  else
    return false;
} // end serialDisplaySlate





void serialSendHello(Serial myPort)
{  
  if (SerialEnabled)
  {
    println("MeggyCom: \tSending Hello to Meggy Jr."); 
    myPort.clear(); 
    myPort.write('h');
  }
  return;
} // end serialSendHello






boolean serialErase(Serial myPort)    // erase Meggy screen
{  
  if (SerialEnabled)
  {  
    myPort.write('c');  
    return true; 
  }
  else
    return false;
} // end serialErase





void serialEvent(Serial myPort) {
  // read a byte from the serial port:
  int inByte = myPort.read();
  // if this is the first byte received, and it's an A,
  // clear the serial buffer and note that you've
  // had first contact from the microcontroller. 
  // Otherwise, add the incoming byte to the array:
  if (ComEstablished == false) {

    //  if (inByte == 255) { // Proper response to Hello char
    myPort.clear();        // clear the serial port buffer
    ComEstablished = true; 
    println("\tMeggy says to say hi to you. :)");
    //  } 

  } 
  else {

    if (inByte == 'X')
      println("\tMeggy Jr reports: command not understood.");
    else if (inByte == 'B')
      println("\tMeggy Jr reports: Bad input values.");
    else if (inByte == 'T')
      println("\tMeggy Jr reports: Timeout error.");

  }
}


 
void charLookup(int letter)    // Font table
{  

  // Design: Liquid Fonts http://www.theiling.de/projects/liquid.html
 
  switch (letter) {

  case ' ':
    SubChar[0] = 0;
    SubChar[1] = 0;
    letter_length = 2;
    break;

  case '!':
    SubChar[0] = 122; 
    letter_length = 1;
    break;

  case '"':
    SubChar[0] = 96;
    SubChar[1] = 0;
    SubChar[2] = 96;
    letter_length = 3;
    break;

  case '#':
    SubChar[0] = 36;
    SubChar[1] = 126;
    SubChar[2] = 36;
    SubChar[3] = 126;
    SubChar[4] = 36;
    letter_length = 5;
    break;

  case '$':
    SubChar[0] = 16;
    SubChar[1] = 42;
    SubChar[2] = 127;
    SubChar[3] = 42;
    SubChar[4] = 4;

    letter_length = 5;
    break;

  case '%':
    SubChar[0] = 36;
    SubChar[1] = 8;
    SubChar[2] = 18;
    SubChar[3] = 0;
    SubChar[4] = 0;
    letter_length = 3;
    break;

  case '&':
    SubChar[0] = 44;
    SubChar[1] = 82;
    SubChar[2] = 42;
    SubChar[3] = 4;
    SubChar[4] = 10;
    letter_length = 5;
    break;



  case '@':
    SubChar[0] = 62;
    SubChar[1] = 65;
    SubChar[2] = 73;
    SubChar[3] = 85;
    SubChar[4] = 60;
    letter_length = 5;
    break;

  case 'A':
    SubChar[0] = 62;
    SubChar[1] = 72;
    SubChar[2] = 62;
    letter_length = 3;
    break;

  case 'B':
    SubChar[0] = 126;
    SubChar[1] = 82;
    SubChar[2] = 44; 
    letter_length = 3;
    break;

  case 'C':
    SubChar[0] = 60;
    SubChar[1] = 66;
    SubChar[2] = 66; 
    letter_length = 3;
    break;

  case 'D':
    SubChar[0] = 126;
    SubChar[1] = 66;
    SubChar[2] = 60; 
    letter_length = 3;
    break;


  case 'E':
    SubChar[0] = 126;
    SubChar[1] = 82;
    SubChar[2] = 66; 
    letter_length = 3;
    break;

  case 'F':
    SubChar[0] = 126;
    SubChar[1] = 80;
    SubChar[2] = 64; 
    letter_length = 3;
    break;

  case 'G':
    SubChar[0] = 60;
    SubChar[1] = 66;
    SubChar[2] = 78; 
    letter_length = 3;
    break;

  case 'H':
    SubChar[0] = 126;
    SubChar[1] = 16;
    SubChar[2] = 126; 
    letter_length = 3;
    break;

  case 'I':
    SubChar[0] = 126;
    letter_length = 1;
    break;

  case 'J':
    SubChar[0] = 1;
    SubChar[1] = 126; 
    letter_length = 2;
    break;

  case 'K':
    SubChar[0] = 126;
    SubChar[1] = 16;
    SubChar[2] = 110; 
    letter_length = 3;
    break;

  case 'L':
    SubChar[0] = 126;
    SubChar[1] = 2;
    SubChar[2] = 2; 
    letter_length = 3;
    break;

  case 'M':
    SubChar[0] = 126;
    SubChar[1] = 32;
    SubChar[2] = 16;
    SubChar[3] = 32;
    SubChar[4] = 126;
    letter_length = 5;
    break;

  case 'N':
    SubChar[0] = 126;
    SubChar[1] = 32;
    SubChar[2] = 16;
    SubChar[3] = 126; 
    letter_length = 4;
    break;

  case 'O':
    SubChar[0] = 60;
    SubChar[1] = 66;
    SubChar[2] = 60; 
    letter_length = 3;
    break;

  case 'P':
    SubChar[0] = 126;
    SubChar[1] = 72;
    SubChar[2] = 48; 
    letter_length = 3;
    break;

  case 'Q':
    SubChar[0] = 60;
    SubChar[1] = 67;
    SubChar[2] = 61; 
    letter_length = 3;
    break;

  case 'R':
    SubChar[0] = 126;
    SubChar[1] = 72;
    SubChar[2] = 54; 
    letter_length = 3;
    break;

  case 'S':
    SubChar[0] = 34;
    SubChar[1] = 82;
    SubChar[2] = 76; 
    letter_length = 3;
    break;

  case 'T':
    SubChar[0] = 64;
    SubChar[1] = 126;
    SubChar[2] = 64; 
    letter_length = 3;
    break;

  case 'U':
    SubChar[0] = 124;
    SubChar[1] = 2;
    SubChar[2] = 126; 
    letter_length = 3;
    break;

  case 'V':
    SubChar[0] = 126;
    SubChar[1] = 4;
    SubChar[2] = 120; 
    letter_length = 3;
    break;

  case 'W':
    SubChar[0] = 124;
    SubChar[1] = 2;
    SubChar[2] = 12;
    SubChar[3] = 2;
    SubChar[4] = 124;
    letter_length = 5;
    break;

  case 'X':
    SubChar[0] = 102;
    SubChar[1] = 24;
    SubChar[2] = 102;
    letter_length = 3;
    break;

  case 'Y':
    SubChar[0] = 112;
    SubChar[1] = 14;
    SubChar[2] = 112;
    letter_length = 3;
    break;

  case 'Z':
    SubChar[0] = 70;
    SubChar[1] = 90;
    SubChar[2] = 98; 
    letter_length = 3;
    break;

  case 'a':
    SubChar[0] = 12;
    SubChar[1] = 18;
    SubChar[2] = 30; 
    letter_length = 3;
    break;

  case 'b':
    SubChar[0] = 126;
    SubChar[1] = 18;
    SubChar[2] = 12; 
    letter_length = 3;
    break;

  case 'c':
    SubChar[0] = 12;
    SubChar[1] = 18; 
    letter_length = 2;
    break;

  case 'd':
    SubChar[0] = 12;
    SubChar[1] = 18;
    SubChar[2] = 126; 
    letter_length = 3;
    break;

  case 'e':
    SubChar[0] = 12;
    SubChar[1] = 22;
    SubChar[2] = 26; 
    letter_length = 3;
    break;

  case 'f':
    SubChar[0] = 62;
    SubChar[1] = 80; 
    letter_length = 2;
    break;

  case 'g':
    SubChar[0] = 13;
    SubChar[1] = 21;
    SubChar[2] = 30; 
    letter_length = 3;
    break;

  case 'h':
    SubChar[0] = 126;
    SubChar[1] = 16;
    SubChar[2] = 14; 
    letter_length = 3;
    break;

  case 'i':
    SubChar[0] = 94;
    letter_length = 1;
    break;

  case 'j':
    SubChar[0] = 1;
    SubChar[1] = 94; 
    letter_length = 2;
    break;

  case 'k':
    SubChar[0] = 126;
    SubChar[1] = 8;
    SubChar[2] = 22; 
    letter_length = 3;
    break;

  case 'l':
    SubChar[0] = 126;
    letter_length = 1;
    break;

  case 'm':
    SubChar[0] = 30;
    SubChar[1] = 16;
    SubChar[2] = 30;
    SubChar[3] = 16;
    SubChar[4] = 14;
    letter_length = 5;
    break;

  case 'n':
    SubChar[0] = 30;
    SubChar[1] = 16;
    SubChar[2] = 14; 
    letter_length = 3;
    break;

  case 'o':
    SubChar[0] = 12;
    SubChar[1] = 18;
    SubChar[2] = 12; 
    letter_length = 3;
    break;

  case 'p':
    SubChar[0] = 31;
    SubChar[1] = 18;
    SubChar[2] = 14; 
    letter_length = 3;
    break;

  case 'q':
    SubChar[0] = 12;
    SubChar[1] = 20;
    SubChar[2] = 31; 
    letter_length = 3;
    break;

  case 'r':
    SubChar[0] = 14;
    SubChar[1] = 16; 
    letter_length = 2;
    break;

  case 's':
    SubChar[0] = 26;
    SubChar[1] = 22; 
    letter_length = 2;
    break;

  case 't':
    SubChar[0] = 60;
    SubChar[1] = 18; 
    letter_length = 2;
    break;

  case 'u':
    SubChar[0] = 28;
    SubChar[1] = 2;
    SubChar[2] = 30; 
    letter_length = 3;
    break;

  case 'v':
    SubChar[0] = 30;
    SubChar[1] = 4;
    SubChar[2] = 24; 
    letter_length = 3;
    break;

  case 'w':
    SubChar[0] = 28;
    SubChar[1] = 2;
    SubChar[2] = 28;
    SubChar[3] = 2;
    SubChar[4] = 28;
    letter_length = 5;
    break;

  case 'x':
    SubChar[0] = 18;
    SubChar[1] = 12;
    SubChar[2] = 18;
    letter_length = 3;
    break;

  case 'y':
    SubChar[0] = 25;
    SubChar[1] = 6;
    SubChar[2] = 24;
    letter_length = 3;
    break;

  case 'z':
    SubChar[0] = 22;
    SubChar[1] = 26; 
    letter_length = 2;
    break;

  case '0':
    SubChar[0] = 60;
    SubChar[1] = 66;
    SubChar[2] = 60; 
    letter_length = 3;
    break;

  case '1':
    SubChar[0] = 32;
    SubChar[1] = 126; 
    letter_length = 2;
    break;

  case '2':
    SubChar[0] = 70;
    SubChar[1] = 74;
    SubChar[2] = 50; 
    letter_length = 3;
    break;

  case '3':
    SubChar[0] = 66;
    SubChar[1] = 82;
    SubChar[2] = 108; 
    letter_length = 3;
    break;

  case '4':
    SubChar[0] = 56;
    SubChar[1] = 8;
    SubChar[2] = 126; 
    letter_length = 3;
    break;

  case '5':
    SubChar[0] = 114;
    SubChar[1] = 82;
    SubChar[2] = 76; 
    letter_length = 3;
    break;

  case '6':
    SubChar[0] = 60;
    SubChar[1] = 82;
    SubChar[2] = 76; 
    letter_length = 3;
    break;

  case '7':
    SubChar[0] = 64;
    SubChar[1] = 78;
    SubChar[2] = 112; 
    letter_length = 3;
    break;

  case '8':
    SubChar[0] = 44;
    SubChar[1] = 82;
    SubChar[2] = 44; 
    letter_length = 3;
    break;

  case '9':
    SubChar[0] = 50;
    SubChar[1] = 74;
    SubChar[2] = 60; 
    letter_length = 3;
    break;

  case ':':
    SubChar[0] = 18; 
    letter_length = 1;
    break;

  case ';':
    SubChar[0] = 1;
    SubChar[1] = 38; 
    letter_length = 2;
    break;

  case '<':
    SubChar[0] = 8;
    SubChar[1] = 20;
    SubChar[2] = 34; 
    letter_length = 3;
    break;

  case '=':
    SubChar[0] = 20;
    SubChar[1] = 20;
    SubChar[2] = 20; 
    letter_length = 3;
    break;

  case '>':
    SubChar[2] = 8;
    SubChar[1] = 20;
    SubChar[0] = 34; 
    letter_length = 3;
    break;

  case '?':
    SubChar[0] = 74;
    SubChar[1] = 80;
    SubChar[2] = 32; 
    letter_length = 3;
    break;

  case '[':
    SubChar[0] = 126;
    SubChar[1] = 66; 
    letter_length = 2;
    break;

  case '\\':
    SubChar[0] = 96;
    SubChar[1] = 24; 
    SubChar[2] = 6; 
    letter_length = 3;
    break;

  case ']':
    SubChar[1] = 126;
    SubChar[0] = 66; 
    letter_length = 2;
    break;

  case '^':
    SubChar[0] = 32;
    SubChar[1] = 64; 
    SubChar[2] = 32; 
    letter_length = 3;
    break;

  case '_':
    SubChar[0] = 2;
    SubChar[1] = 2; 
    SubChar[2] = 2; 
    letter_length = 3;
    break;

  case '{':
    SubChar[0] = 16;
    SubChar[1] = 110; 
    SubChar[2] = 66; 
    letter_length = 3;
    break;

  case '|':
    SubChar[0] = 126; 
    letter_length = 1;
    break;

  case '}':
    SubChar[2] = 16;
    SubChar[1] = 110; 
    SubChar[0] = 66; 
    letter_length = 3;
    break;

  case '~':
    SubChar[0] = 32;
    SubChar[1] = 64; 
    SubChar[2] = 32; 
    SubChar[3] = 64; 
    letter_length = 4;
    break;

  case '\'':
    SubChar[0] = 96; 
    letter_length = 1;
    break; 

  case '`':
    SubChar[0] = 64;
    SubChar[1] = 32; 
    letter_length = 2;
    break;

  case '(':
    SubChar[0] = 60;
    SubChar[1] = 66; 
    letter_length = 2;
    break;

  case ')':
    SubChar[1] = 60;
    SubChar[0] = 66; 
    letter_length = 2;
    break;

  case '*':
    SubChar[0] = 42;
    SubChar[1] = 28; 
    SubChar[2] = 42; 
    letter_length = 3; 
    break;

  case ',': 
    SubChar[0] = 1; 
    SubChar[1] = 6;  
    letter_length = 2; 
    break;

  case '-': 
    SubChar[0] = 8; 
    SubChar[1] = 8;  
    letter_length = 2; 
    break;
 
  case '+': 
    SubChar[0] = 8; 
    SubChar[1] = 28;  
    SubChar[2] = 8;  
    letter_length = 3; 
    break;
 
  case '.': 
    SubChar[0] = 2;
    letter_length = 1;
    break;

  case '/':
    SubChar[2] = 96;
    SubChar[1] = 24; 
    SubChar[0] = 6; 
    letter_length = 3;
    break;
 
  case '\n':
    SubChar[2] = 0; 
    SubChar[1] = 0;  
    letter_length = 2;
    break;
 
  default:      // Unknown characters: Display as a rectangle.
    SubChar[0] = 126; 
    SubChar[1] = 126;  
    SubChar[2] = 126;  
    letter_length = 3;
    break; 
  }

} // end serialErase



